/* Implementation of polynom operations
 * Author:  Yotam Medini  yotam.medini@gmail.com -- Created: 2010/December/20 
 */

#include "polynom.h"
#include <stdlib.h>
#include <math.h>


const double polynom_epsilon = 1e-6;


polynom_t  *polynom_create(int degree)
{
    int  k;
    polynom_t  *p = malloc(sizeof(polynom_t));
    p->degree = degree;
    p->coefs = malloc((degree + 1)*sizeof(double));
    for (k = 0; k < degree; k++) { p->coefs[k] = 0; }
    p->coefs[degree] = 1;
    return p;
}


polynom_t  *polynom_create_constant(double c0)
{
    polynom_t  *p = polynom_create(0);
    p->coefs[0] = c0;
    return p;
}


void polynom_copy(polynom_t *target, polynom_t *source)
{
    int  i, d = source->degree;
    target->degree = d;
    target->coefs = realloc(target->coefs, (d + 1) * sizeof(double));
    for (i = 0;  i < d;  i++)
    {
        target->coefs[i] = source->coefs[i];
    }
}


void polynom_destroy(polynom_t *poly)
{
    free(poly->coefs);
    free(poly);
}


double polynom_coef(polynom_t *p, int i)
{
    return (i <= p->degree ? p->coefs[i] : 0.);
}


void polynom_print(FILE *f, polynom_t *p)
{
    int  k = p->degree;

    fprintf(f, "%gx^%d", p->coefs[k], k);
    for (k--;  k >= 0;  k--)
    {
        double  a = p->coefs[k];
        char    *sym = "+";
        if (a < 0)
        {
            a = -a;
            sym = "-";
        }
        fprintf(f, " %s %gx^%d", sym, a, k);
    }
}

void polynom_reduce(polynom_t *p)
{
    /* HomeWork ... */
}


void polynom_add(polynom_t *result, polynom_t *p, polynom_t *q)
{
    /* HomeWork ... */
}


void polynom_subtract(polynom_t *result, polynom_t *p, polynom_t *q)
{
    polynom_t  *minus1 = polynom_create_constant(-1);
    polynom_t  *minus_q = polynom_create(0);
    polynom_multiply(minus_q, minus1, q);

    polynom_add(result, p, minus_q);

    polynom_destroy(minus1);
    polynom_destroy(minus_q);
}


void polynom_multiply(polynom_t *result, polynom_t *p, polynom_t *q)
{
    /* HomeWork ... */
}
