/* Author:  Yotam Medini  yotam.medini@gmail.com -- Created: 2010/December/19 */
#if !defined(_POLYNOM_H)

#define _POLYNOM_H

#include <stdio.h>  /* Just for FILE definition */

/* Constant small number. Smaller coefficients are considered to be zero */
extern const double  polynom_epsilon;

typedef struct
{
    int     degree;
    double  *coefs; /* (degree+1) coefficients starting from the constnt */
} polynom_t;


/* Create a polynomn of degree n. Leading coefficient 1, the rest are 0. */
extern polynom_t  *polynom_create(int degree);

/* Convenient, create a constant polynomial. p(x) = c0 */
extern polynom_t  *polynom_create_constant(double c0);

/* Destroy a polynom */
extern void polynom_destroy(polynom_t *poly);

/* Return the [i]-th coefficient, or zero if it does not exist */
extern double polynom_coef(polynom_t *p, int i);

/* Copy a polynom, both are already created */
extern void polynom_copy(polynom_t *target, polynom_t *polynom_source);

/* Print polynom to a file.
 * Format example: 7.8x^3 + 6.7x^2 + 5.6x + 4.5 
 */
extern void polynom_print(FILE *f, polynom_t *p);

/*  Set to zero, all coefficients whose absolute value is <= polynom_epsilon.
 *  If the p becomes zero, then set the degree to zero.
 *  Otherwise, reduce the degree so the leading coefficient will be non-zero,
 */
extern void polynom_reduce(polynom_t *p);

/* Add:  result := p + q,  and reduce the sum  */
extern void polynom_add(polynom_t *result, polynom_t *p, polynom_t *q);

/* Add:  result := p - q,  and reduce the sum  */
extern void polynom_subtract(polynom_t *result, polynom_t *p, polynom_t *q);

/* Multiply  result := p x q */
extern void polynom_multiply(polynom_t *result, polynom_t *p, polynom_t *q);

#endif
